#include <FastLED.h>
#define NUM_LEDS 17
#define NUM_LEDS_BODY 41
#define NUM_QUILLS 6
#define BRIGHTNESS  200
#define COOLING  20
#define SPARKING 20
#define FRAMES_PER_SECOND 40
bool gReverseDirection = false;

#define DATA_PIN_1 7
#define DATA_PIN_2 8
#define DATA_PIN_3 9
#define DATA_PIN_4 10
#define DATA_PIN_5 11
#define DATA_PIN_6 12
#define DATA_PIN_7 6

#define BUTTON_PIN_2 2
#define BUTTON_PIN_1 3

#define LED_OPERATION_1 1
#define LED_OPERATION_2 2

CRGB leds1[NUM_LEDS];
CRGB leds2[NUM_LEDS];
CRGB leds3[NUM_LEDS];
CRGB leds4[NUM_LEDS];
CRGB leds5[NUM_LEDS];
CRGB leds6[NUM_LEDS];

CRGB leds7[NUM_LEDS_BODY];

int POS[] = {0,3,6,9,12,15};
int BODY_POS;
int STATUS;
bool update;

void setup() {
  update = false;
    delay(2000);
    STATUS = 0;
    BODY_POS = 0;
    attachInterrupt(digitalPinToInterrupt(BUTTON_PIN_1), changeState_1, RISING);
    attachInterrupt(digitalPinToInterrupt(BUTTON_PIN_2), changeState_2, RISING);

    FastLED.addLeds<WS2811, DATA_PIN_1, RGB>(leds1, NUM_LEDS);
    FastLED.addLeds<WS2811, DATA_PIN_2, RGB>(leds2, NUM_LEDS);
    FastLED.addLeds<WS2811, DATA_PIN_3, RGB>(leds3, NUM_LEDS);
    FastLED.addLeds<WS2811, DATA_PIN_4, RGB>(leds4, NUM_LEDS);
    FastLED.addLeds<WS2811, DATA_PIN_5, RGB>(leds5, NUM_LEDS);
    FastLED.addLeds<WS2811, DATA_PIN_6, RGB>(leds6, NUM_LEDS);

    FastLED.addLeds<WS2811, DATA_PIN_7, RGB>(leds7, NUM_LEDS_BODY);
}

void loop() {
  switch(STATUS) {
    case LED_OPERATION_1:
      led_program_1();
      break;
    case LED_OPERATION_2:
      led_program_2();
      break;
    default:
      clear_leds();
  }
}

void changeState_1() {
  if(!update){
    if(STATUS == LED_OPERATION_1) {
      STATUS = 0;
    }
    else {
      STATUS = LED_OPERATION_1;
      Serial.println("**1");
    }
    update = true;
  }
  else {
    update = false;
  }
}

void changeState_2() {
  if(!update){
    if(STATUS == LED_OPERATION_2) {
      STATUS = 0;
    }
    else {
      STATUS = LED_OPERATION_2;
      Serial.println("**2");
    }
    update = true;
  }
  else {
    update = false;
  }
}

void clear_leds() {
  for(int ledloc = 0; ledloc < NUM_LEDS; ledloc = ledloc + 1) {
      leds1[ledloc] = CRGB::Black;
      leds2[ledloc] = CRGB::Black;
      leds3[ledloc] = CRGB::Black;
      leds4[ledloc] = CRGB::Black;
      leds5[ledloc] = CRGB::Black;
      leds6[ledloc] = CRGB::Black;
  }

  for(int ledloc = 0; ledloc < NUM_LEDS_BODY; ledloc = ledloc + 1) {
    leds7[ledloc] = CRGB::Black;
  }
  FastLED.show();
}

void led_program_2() {
  static uint8_t heat[NUM_QUILLS][NUM_LEDS];
  static uint8_t bodyHeat[NUM_LEDS_BODY];

  // Step 1.  Cool down every cell a little
  for( int j = 0; j < NUM_QUILLS; j++) {
    for( int i = 0; i < NUM_LEDS; i++) {
      heat[j][i] = qsub8( heat[j][i],  random8(0, ((COOLING * 10) / NUM_LEDS) + 2));
    }
  }

  for( int i = 0; i < NUM_LEDS_BODY; i++) {
    bodyHeat[i] = qsub8( bodyHeat[i],  random8(0, ((COOLING * 10) / NUM_LEDS_BODY) + 2));
  }

  // Step 2.  Heat from each cell drifts 'up' and diffuses a little
  for( int j = 0; j < NUM_QUILLS; j++) {
    for( int k= NUM_LEDS - 1; k >= 2; k--) {
      heat[j][k] = (heat[j][k - 1] + heat[j][k - 2] + heat[j][k - 2] ) / 3;
    }
  }

  for( int k= NUM_LEDS_BODY - 1; k >= 2; k--) {
    bodyHeat[k] = (bodyHeat[k - 1] + bodyHeat[k - 2] + bodyHeat[k - 2] ) / 3;
  }
  
  // Step 3.  Randomly ignite new 'sparks' of heat near the bottom
  if( random8() < SPARKING ) {
    for( int j = 0; j < NUM_QUILLS; j++) {
      int y = random8(NUM_LEDS);
      heat[j][y] = qadd8( heat[j][y], random8(160,255) );
    }

    int y = random8(NUM_LEDS_BODY);
    bodyHeat[y] = qadd8( bodyHeat[y], random8(160,255) );
  }

  // Step 4.  Map from heat cells to LED colors
  for( int j = 0; j < NUM_QUILLS; j++) {
    for( int k = 0; k < NUM_LEDS; k++) {
      CRGB color = HeatColor( heat[j][k]);
      
      int pixelnumber;
      if( gReverseDirection ) {
        pixelnumber = (NUM_LEDS-1) - k;
      } else {
        pixelnumber = k;
      }

      switch(j) {
        case 0:
          leds1[pixelnumber] = color;
          break;
        case 1:
          leds2[pixelnumber] = color;
          break;
        case 2:
          leds3[pixelnumber] = color;
          break;
        case 3:
          leds4[pixelnumber] = color;
          break;
        case 4:
          leds5[pixelnumber] = color;
          break;
        case 5:
          leds6[pixelnumber] = color;
          break;
      }
    }
  }

  for( int k = 0; k < NUM_LEDS_BODY; k++) {
    CRGB color = HeatColor( bodyHeat[k]);
    
    int pixelnumber;
    if( gReverseDirection ) {
      pixelnumber = (NUM_LEDS_BODY-1) - k;
    } else {
      pixelnumber = k;
    }

    leds7[pixelnumber] = color;
  }
  FastLED.show();
  // FastLED.delay(1000 / FRAMES_PER_SECOND);
}

void led_program_1() {
   for(int whiteLed = 0; whiteLed < NUM_LEDS; whiteLed = whiteLed + 1) {
      leds1[POS[0]] = CRGB::White;
      leds2[POS[1]] = CRGB::White;
      leds3[POS[2]] = CRGB::White;
      leds4[POS[3]] = CRGB::White;
      leds5[POS[4]] = CRGB::White;
      leds6[POS[5]] = CRGB::White;

      leds7[BODY_POS] = CRGB::White;

      // Show the leds (only one of which is set to white, from above)
      FastLED.show();

      // Wait a little bit
      delay(10);

      // Turn our current led back to black for the next loop around
      leds1[POS[0]] = CRGB::Black;
      leds2[POS[1]] = CRGB::Black;
      leds3[POS[2]] = CRGB::Black;
      leds4[POS[3]] = CRGB::Black;
      leds5[POS[4]] = CRGB::Black;
      leds6[POS[5]] = CRGB::Black;

      leds7[BODY_POS] = CRGB::Black;

      for(int i = 0; i < 6; i++) {
        if(POS[i] >= NUM_LEDS - 1) {
          POS[i] = 0;
        }
        else {
          POS[i]++;
        }
      }

      if(BODY_POS + 1 == NUM_LEDS_BODY) {
        BODY_POS = 0;
      } 
      else {
        BODY_POS++;
      }
   }
}
